from abc import abstractclassmethod
from .streaming_service import StreamingService
from bdaserviceutils import get_kafka_binder_brokers, get_input_channel, get_output_channel
from kafka import KafkaConsumer, KafkaProducer
from abc import ABC, abstractclassmethod
import json


class ProcessorService(ABC, StreamingService):

    alida_service_mode = "processor"
    
    def __init__(self):
        super().__init__()
        self.consumer = KafkaConsumer(get_input_channel(), bootstrap_servers=[get_kafka_binder_brokers()])
        self.producer = KafkaProducer(bootstrap_servers=[get_kafka_binder_brokers()])

    def run(self):
        for message in self.consumer:
            json_message = json.loads(message.value.decode('utf-8'))
            self.on_message(message=json_message)

    @abstractclassmethod
    def on_message(self, message):
        pass

    def send_message(self, message):
        self.producer.send(get_output_channel(), json.dumps(message).encode('utf-8'))



import os

class IO:
    @staticmethod
    def dataset_to_dataset(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            resulting_dataset = func(self, dataset)
            self.save_dataset(resulting_dataset)

        return wrapper


    @staticmethod
    def dataset_to_model(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            model_path = func(self, dataset)
            self.upload_model(model_path)

        return wrapper


    @staticmethod
    def dataset_and_model_to_dataset(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            model_path = os.path.join(".", "model")
            self.download_model(path=model_path)
            resulting_dataset = func(self, dataset, model_path)
            self.save_dataset(resulting_dataset)

        return wrapper


    @staticmethod
    def dataset_and_model_to_dataset_and_model(func):
        
        def wrapper(*args):
            self = args[0]
            dataset = self.get_dataset()
            model_path = os.path.join(".", "model")
            self.download_model(path=model_path)
            resulting_dataset, resulting_model = func(self, dataset, model_path)
            self.save_dataset(resulting_dataset)
            self.upload_model(resulting_model)

        return wrapper




from .generic_service import GenericService
from bdaserviceutils import HttpHealthServer, get_kafka_binder_brokers, get_input_channel, get_output_channel

class StreamingService(GenericService):

    def __init__(self):
        super().__init__()
        HttpHealthServer.run_thread()



from .spark_service import SparkService
import os
import glob
import pandas as pd

class PandasService(SparkService):
    def __init__(self):
        super().__init__()

    def get_dataset(self):
        spark_df = super().get_dataset().toPandas()

        # Save dataset to tmp file and read it using pandas
        path = "/tmp"
        spark_df.repartition(1).write.format("com.databricks.spark.csv").option("header", "false").save(path)
        os.chdir(path)
        result = glob.glob('*.{}'.format("csv"))
        return pd.read_csv(result[0])


    def save_dataset(self, dataset):
        super().save_dataset(self.spark.createDataFrame(dataset))
